﻿package implement

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/dao"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	zhios_order_relate_utils "code.fnuoos.com/EggPlanet/egg_models.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/EggPlanet/egg_models.git/utils/logx"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

func NewUserWalletFlowDb(engine *xorm.Engine) dao.UserWalletFlowDao {
	return &UserWalletFlowDb{Db: engine}
}

type UserWalletFlowDb struct {
	Db *xorm.Engine
}

func (u UserWalletFlowDb) UserWalletFlowInsertBySession(session *xorm.Session, userWalletFlow *model.UserWalletFlow) (int64, error) {
	_, err := session.InsertOne(userWalletFlow)
	if err != nil {
		return 0, err
	}
	return userWalletFlow.Id, nil
}

func (u UserWalletFlowDb) UserWalletFlowFindByParams(params map[string]interface{}, page, limit int) (*[]model.UserWalletFlow, int64, error) {
	var m []model.UserWalletFlow
	if page == 0 || limit == 0 {
		page = 1
		limit = 10
	}
	var total int64
	var err error
	if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
		//指定In查询
		if total, err = u.Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(limit, (page-1)*limit).FindAndCount(&m); err != nil {
			return nil, 0, zhios_order_relate_logx.Warn(err)
		}
	} else {
		var query = fmt.Sprintf("%s =?", params["key"])
		total, err = u.Db.Where(query, params["value"]).Limit(limit, (page-1)*limit).FindAndCount(&m)
		if err != nil {
			return nil, 0, zhios_order_relate_logx.Error(err)
		}
	}
	return &m, total, nil
}

func (u UserWalletFlowDb) UserWalletFlowFindByCoinAndUser(page, pageSize int, uid int64, startAt string, endAt string, direction int, isAsc bool, kind int, state int) ([]model.UserWalletFlow, int64, error) {
	var m []model.UserWalletFlow
	session := u.Db.Where("uid = ?", uid)
	if startAt != "" && endAt != "" {
		session = session.And("create_at >= ?", startAt).And("create_at <= ?", endAt)
	}
	if direction != 0 {
		session = session.And("direction = ?", direction)
	}
	if kind != 0 {
		session = session.And("kind = ?", kind)
	}
	if state != 0 {
		session = session.And("state = ?", state)
	}
	session = session.Limit(pageSize, (page-1)*pageSize)
	if isAsc {
		session = session.Asc("create_at")
	} else {
		session = session.Desc("create_at")
	}
	total, err := session.FindAndCount(&m)
	if err != nil {
		return nil, 0, err
	}

	return m, total, nil
}
